function main(method,n,alpha,component,id)
% INPUTS
% method   : type of CI, 'KMS' or 'AS'
% n        : sample size
% alpha    : nominal level (coverage = 1-alpha)
% component: parameter componnent of interest
% id       : 'point' or 'partial', distinguishes two DGPs

% Set model parameters
Nmc = 1000;                         % # of MC replications
M   = 2001;                         % # of bootstrap replications
ki  = sqrt(log(n));                 % GMS parameter
Beta = 0.01;                        % amount of conservativism for rho
selp = 0.5;                 		% equilibrium selection probability
[z,x,dz,pz,theta_true,lx,ux,p]...
 = set_specification(component,id);	% DGP (covariate support, true parameter, parameter space & p)
d = length(theta_true);             % dimension of parameter space
beta1_true = theta_true(1:dz(1));   % true parameter values
beta2_true = theta_true(dz(1)+1:sum(dz(1:2)));
delta1_true = theta_true(sum(dz(1:2))+1:sum(dz(1:3)));
delta2_true = theta_true(sum(dz(1:3))+1:end);
Nz = size(z,1);                     % # of support points
Jeqraw = 2;                         % # of equalities (w/o conditioning)
Jineqraw = 2;                       % # of inequalities (w/o conditioning)
Jraw = [Jeqraw, Jineqraw];          %
Jeq  = Jeqraw*Nz;                   %
Jineq= Jineqraw*Nz;                 %
J = 2*Jeq + Jineq;                  % tatal # of inequalities
rho = get_rho(Beta,J,d);            % size of the rho-box
type = 'two-sided';					% type of CI
algorithm = 'sqp';                  % algorithm used by fmincon


% Set parameters for computation
siglb = 1e-4;                       % lower bound on sigma
gpu = 'off';                        % set to 'on' if you use GPUs
parallel = 'on';					% set to 'on' if you use Parallel Computing Toolbox
mbase = 20;                         % 
minit = d*mbase + 1; 				% # of initial evaluation points
madd = mbase;                       % # of additional points to be sampled in each M-step
niter = 30;                         % maximum # of iterations for EAM steps

% Initialize Objects
stream = RandStream('mlfg6331_64','Seed',0);                     % initialize random number generator
RandStream.setGlobalStream(stream)                               %
addpath ./dace                                                   % use DACE kriging toolbox
dace_theta = 10*ones(1,d);                                       % set parameters for the A-step
dace_lob = 1e-1*ones(1,d);                                       %
dace_upb = 20*ones(1,d);                                         %
if strcmp(gpu,'on')                                              %
    dev = gpuDevice();                                           % initialize GPUs
end          

% Get the population upper bound
[~,~,pint_eq,pint_ineq]=population_freq(theta_true,z,dz,pz,Jraw,selp);
options_fmincon = optimoptions('fmincon','Display','off','GradObj','on','GradConstr',...
    'on','Algorithm',algorithm,'TolCon',1e-6,'MaxFunEvals',3000,'MaxIter',1000);
nonlcon_pop = @(theta)population_nonlcon(theta,theta_true,z,dz,pz,Jraw,selp);
[x1_U,fvl1_U,exitflag1_U,output1_U,lambda1_U,grad1_U,hessian1_U] ...
    = population_proj(theta_true,p,lx,ux,nonlcon_pop,options_fmincon);
[x1_L,fvl1_L,exitflag1_L,output1_L,lambda1_L,grad1_L,hessian1_L] ...
    = population_proj(theta_true,-p,lx,ux,nonlcon_pop,options_fmincon);
if exitflag1_U > 0					%
    theta_U = x1_U(p~=0);			% upper bound of projection
else                                %
    theta_U = theta_true(p~=0);     %
end                                 %
                                    %
if exitflag1_L > 0                  %
    theta_L = x1_L(p~=0);           % lower bound of projection
else                                %
    theta_L = theta_true(p~=0);     %
end                                 %
%

% Data Generation %
[X1,X2,U1,U2,V] = drawsample(n,Nmc,selp,z,dz,pz);	             % generate X, U, and V (used for eqbm seleciton)
W1 = X1;                                                         % 
W2 = X2;                                                         %
X1V = get_nonconstantx(X1);                                      % get covariates that are not constant
X2V = get_nonconstantx(X2);                                      %
Xb1 = squeeze(sum(X1.*repmat(beta1_true',[n,1,Nmc]),2));         % get mean utilities w/o iteraction effects
Xb2 = squeeze(sum(X2.*repmat(beta2_true',[n,1,Nmc]),2));         % 
Wd1 = squeeze(sum(W1.*repmat(delta1_true',[n,1,Nmc]),2));        % get interaction effects
Wd2 = squeeze(sum(W2.*repmat(delta2_true',[n,1,Nmc]),2));        %

if strcmp(gpu,'on')
    outcomeFcn = @outcome;                                                  %
    GXb1 = gpuArray(Xb1);                                                   %
    GXb2 = gpuArray(Xb2);                                                   %
    GWd1 = gpuArray(Wd1);                                                   %
    GWd2 = gpuArray(Wd2);                                                   %
    GU1 = gpuArray(U1);                                                     %
    GU2 = gpuArray(U2);                                                     %
    GV  = gpuArray(V);                                                      %
    GY = arrayfun(outcomeFcn,GXb1,GXb2,GWd1,GWd2,GU1,GU2,GV); 				% generate outcome across all MC replications
    Y = gather(GY);                                                         %
elseif strcmp(gpu,'off')                                                    %
    Y = arrayfun(@outcome,Xb1,Xb2,Wd1,Wd2,U1,U2,V);      				    %
end                                                                         %
phat = emp_freq(Y,X1V,X2V,Jraw,x);                                          % compute empirical frequencies
phateq = phat(1:Jeq,:);                                                     % for equalities
phatineq = phat(Jeq+1:end,:);                                               % and for inequalities

% Bootstrap everything %
emp_freqFcn = @(Y,X1V,X2V) emp_freq(Y,X1V,X2V,Jraw,x);
if strcmp(parallel,'on')
    opt = statset('UseParallel',true,'UseSubstreams',true);
    temp = bootstrp(M,emp_freqFcn,...
        Y,X1V,X2V,'Options',opt);                                           % bootstrap data to get empirical freuencieis
elseif strcmp(parallel,'off')
    temp = bootstrp(M,emp_freqFcn,...
        Y,X1V,X2V);  		                                                % bootstrap data to get empirical freuencieis
end
temp2 = reshape(temp,M,Jeq+Jineq,Nmc);
temp2 = permute(temp2,[2,1,3]);
bdat = reshape(temp2,Jeq+Jineq,M*Nmc);                                      %
bphateq = bdat(1:Jeq,:);                                                    % (Jeq x M) bootstrapped frequencies are tiled Nmc times
bphatineq = bdat(Jeq+1:end,:);                                              % (Jineq x M) bootstrapped frequencies are tiled Nmc times

% Obtain Std Devs
[sigeq, sigineq] =  get_sigma(phateq,phatineq);
sigeq = max(siglb,sigeq);                                                   % regularize sigma if too close to 0
sigineq = max(siglb,sigineq);
sig = [sigeq;sigineq];

% Obtain Inputs into LP that do not depend on theta
pint_eq = [pint_eq;pint_eq];                                                % indicators for predicted probabilities in (0,1) (off boundary)
[ehat_eq,ehat_ineq,int_eq,int_ineq]...                                      % empirical frequencies
    = get_moment(phateq,phatineq);                                          %
ehat = [ehat_eq;ehat_ineq];                                                 %
int_proj  = [int_eq;int_ineq].*repmat([pint_eq;pint_ineq],1,Nmc);           % indicators for empirical frequencies and predicted probabilties in (0,1)
[bhat_eq,bhat_ineq] = get_moment(bphateq,bphatineq);                        % 
Gmomeq = sqrt(n).*(bhat_eq - repelem(ehat_eq,1,M)).*repmat(pint_eq,1,M*Nmc);%
Gmomineq = sqrt(n).*(bhat_ineq - repelem(ehat_ineq,1,M)).*repmat(pint_ineq,1,M*Nmc);
G = [Gmomeq./repelem(sigeq,1,M);Gmomineq./repelem(sigineq,1,M)];			% Re-centered bootstrrapped empirical process

% Draw Initial Evaluation Points %
Theta_init = draw_points(minit,d,lx,ux);									% draw initial evaluation points
if ~isequal(x1_U,x1_L)
    Theta_init = [x1_U';x1_L';Theta_init];
else
    Theta_init = [x1_U';Theta_init];
end
Theta_U_add = zeros(madd*niter,d);                                          % initialize matrix for additional points (for upper bound)
Theta_L_add = zeros(madd*niter,d);                                          % similarly for lower bound
C_U_add = zeros(madd*niter,1);                                              % initialize vector for cvs (for upper bound)
C_L_add = zeros(madd*niter,1);   											% similarly for lower bound

% Beginning of MC
c_at_xu = zeros(Nmc,1);					% cv at x_U (argmax for population projection)
c_at_xl = zeros(Nmc,1);                 % cv at x_L (argmin for population projection)
phi_at_xu = zeros(J,Nmc);               % GMS at x_U
phi_at_xl = zeros(J,Nmc);               % GMS at x_L
that_U = zeros(Nmc,1);					% vector to store endpoints of CI
that_L = zeros(Nmc,1);	                %
x_that_U = zeros(Nmc,d);                % argmax of sample projection
x_that_L = zeros(Nmc,d);                % argmin of sample projection
steps = zeros(Nmc,1);					% vector to store # of steps until convergence
Neval = zeros(Nmc,2);                   % # of evaluation points
optim_val_U = NaN(niter,Nmc);           % matrix to store upper bounds over EAM iterations
optim_val_L = NaN(niter,Nmc);           %
optim_flag_U = NaN(niter,Nmc);			% matrix to store exitflags over EAM iterations
optim_flag_L = NaN(niter,Nmc);			% 
h_init = 0.5;                 	        % initial width of a band to which we add points
outer_tol = 1/(sqrt(n)*5);              % tolerance to exit iterations
e_step_time = zeros(Nmc,1);             % total time spent for E-step
a_step_time = zeros(Nmc,1);             % total time spent for A-step
m_step_time = zeros(Nmc,1);             % total time spent for M-step
iter_min = 2;                           % minimum # of EAM iterations

disp('Model Parameters')
disp(['Method        : ' method])
disp(['Sample Size   : ' num2str(n)])
disp(['Nominal Level : ' num2str(alpha)])
disp(['Component     : ' num2str(component)])
disp(['Type of ID    : ' id])
disp(['MC started. # of MC replications = ' num2str(Nmc) '.'])
for s=1:Nmc
    Gs = G(:,(s-1)*M+1:s*M);		    % extract G for this MC rep
    ehats = ehat(:,s);                  % empirical frequency for s-th MC replication                                        
    sigs  = sig(:,s);                   % standard deviations                                         
    ints  = int_proj(:,s);              % indicators for being in (0,1)
    e_eq = ehat_eq(:,s);                % empirical frequency in equality constraitns
    e_ineq = ehat_ineq(:,s);            % empirical frequency in inequality constraints
    s_eq = sigeq(:,s);                  % sigma for equality constraints
    s_ineq = sigineq(:,s);              % sigma for inequality constraints
    theta_U_iter = NaN(niter,1);        % upper bound of CI from each iteration
    theta_L_iter = NaN(niter,1);        % lower bound of CI from each iteration
    eam_U_conv = 0;                     % =1 once upper bound converges
    eam_L_conv = 0;                     % =1 once lower bound converges
    te = zeros(niter,1);                % time spent for E-step in each iteration
    ta = zeros(niter,1);                % time spent for A-step in each iteration
    tm = zeros(niter,1);                % time spent for M-step in each iteration
    flag_temp_U = NaN(niter,1);         % exitflag for upper bound optimization 
    flag_temp_L = NaN(niter,1);         % exitflag for lower bound optimization
    for iter=1:niter
        % Set evaluation points
        if iter==1
            Theta = Theta_init;			% initial evaluation points
        else
            if eam_U_conv == 0 && eam_L_conv == 0  							% Case 1: Either UB or LB not converged yet
                Theta = [Theta_U_add(madd*(iter-2)+1:madd*(iter-1),:)       % add evaluation points for UB and LB
                    Theta_L_add(madd*(iter-2)+1:madd*(iter-1),:)];          % 
            elseif eam_U_conv == 1 && eam_L_conv == 0                       % Case 2: UB converged, LB not yet
                Theta = Theta_L_add(madd*(iter-2)+1:madd*(iter-1),:);       % add evaluation points for LB
            elseif eam_U_conv == 0 && eam_L_conv == 1                       % Case 3: UB not yet, LB converged
                Theta = Theta_U_add(madd*(iter-2)+1:madd*(iter-1),:);       % add evaluation points for UB
            end                                                             %
        end
        
        % E-Step (Evaluate critical values over evaluation points: Theta)
        tic;
         C_temp = E_step(Theta,n,Jraw,z,dz,pz,Gs,e_eq,...
              e_ineq,s_eq,s_ineq,pint_eq,pint_ineq,ki,alpha,rho,p,type,method);                                                                 
        te(iter)=toc;                                                                          
         
        % A-Step (Approximate (Interpolate) critical values outside evaluation points)
        tic
        if iter==1                                                                   
            C_init = C_temp;																   % cvs at initial evaluation points
            dmodel_init = A_step(Theta_init,C_init,dace_theta,dace_lob,dace_upb);              % fit a model using DACE kriging toolbox 
        else                                                                                   %
            if eam_U_conv == 0 && eam_L_conv == 0%                                             % Case 1: Either UB or LB not converged yet
                C_U_add(madd*(iter-2)+1:madd*(iter-1)) = C_temp(1:madd);                       % critical values at additional eval points (UB)
                C_L_add(madd*(iter-2)+1:madd*(iter-1)) = C_temp(madd+1:end); 				   % critical values at additional eval points (LB)
                Theta_U_current = [Theta_init;Theta_U_add(1:madd*(iter-1),:)];                 % current evaluation points (UB)
                Theta_L_current = [Theta_init;Theta_L_add(1:madd*(iter-1),:)];                 % current evaluation points (LB)
                C_U_current = [C_init;C_U_add(1:madd*(iter-1),:)];                             % cvs at current evaluation points (UB)
                C_L_current = [C_init;C_L_add(1:madd*(iter-1),:)];                             % cvs at current evaluation points (LB)
                dmodel_L = A_step(Theta_L_current,C_L_current,dace_theta,dace_lob,dace_upb);   % fit a model using DACE kriging toolbox (UB)
                dmodel_U = A_step(Theta_U_current,C_U_current,dace_theta,dace_lob,dace_upb);   % fit a model using DACE kriging toolbox (LB)
            elseif eam_U_conv == 1 && eam_L_conv == 0                                          % Case 2: UB converged, LB not yet
                C_L_add(madd*(iter-2)+1:madd*(iter-1)) = C_temp;                               %
                Theta_L_current = [Theta_init;Theta_L_add(1:madd*(iter-1),:)];                 %
                C_L_current = [C_init;C_L_add(1:madd*(iter-1),:)];                             %
                dmodel_L = A_step(Theta_L_current,C_L_current,dace_theta,dace_lob,dace_upb);   %
            elseif eam_U_conv == 0 && eam_L_conv == 1                                          % Case 3: UB not yet, LB converged
                C_U_add(madd*(iter-2)+1:madd*(iter-1)) = C_temp;                               %
                Theta_U_current = [Theta_init;Theta_U_add(1:madd*(iter-1),:)];                 %
                C_U_current = [C_init;C_U_add(1:madd*(iter-1),:)];                             %
                dmodel_U = A_step(Theta_U_current,C_U_current,dace_theta,dace_lob,dace_upb);   %
            end%
        end                                                                                                                         
        ta(iter)=toc;                                                                          
        

        % M-Step (Maximize/Minimize p'theta s.t. relaxed moment conditions)
        tic
        if iter==1   																					% 
            theta0_U = set_initial_value(theta_true,x1_U,exitflag1_U);                                  % set initial value for optimization (UB)
            theta0_L = set_initial_value(theta_true,x1_L,exitflag1_L);                                  %
            nonlcon_samp_init = @(theta)nonl_const(theta,dmodel_init,ehats,sigs,ints,n,Jraw,z,dz,pz);   % set relaxed constraints using a surrogate model
            [opt_val_U,x2_U,fvl2_U,exitflag2_U,output2_U,lambda2_U,grad2_U,hessian2_U]...               % calculate UB
                = M_step(theta0_U,nonlcon_samp_init,lx,ux,p,options_fmincon);                           %
            [opt_val_L,x2_L,fvl2_L,exitflag2_L,output2_L,lambda2_L,grad2_L,hessian2_L]...               % calculate LB
                = M_step(theta0_L,nonlcon_samp_init,lx,ux,-p,options_fmincon);                          %
        else                                                                                            %
            if eam_U_conv == 0 && eam_L_conv == 0                                                       % Case 1: Either UB or LB not converged yet
                theta0_U = set_initial_value(x1_U,x2_U,exitflag2_U);                              		%
                theta0_L = set_initial_value(x1_L,x2_L,exitflag2_L);                              		%
                nonlcon_samp_U = @(theta)nonl_const(theta,dmodel_U,ehats,sigs,ints,n,Jraw,z,dz,pz);     %
                nonlcon_samp_L = @(theta)nonl_const(theta,dmodel_L,ehats,sigs,ints,n,Jraw,z,dz,pz);     %
                [opt_val_U,x2_U,fvl2_U,exitflag2_U,output2_U,lambda2_U,grad2_U,hessian2_U]...           %
                    = M_step(theta0_U,nonlcon_samp_U,lx,ux,p,options_fmincon);                          %
                [opt_val_L,x2_L,fvl2_L,exitflag2_L,output2_L,lambda2_L,grad2_L,hessian2_L]...           %
                    = M_step(theta0_L,nonlcon_samp_L,lx,ux,-p,options_fmincon);                         %
            elseif eam_U_conv == 1 && eam_L_conv == 0                                                   % Case 2: UB converged, LB not yet
                theta0_L = set_initial_value(x1_L,x2_L,exitflag2_L);                              		%
                nonlcon_samp_L = @(theta)nonl_const(theta,dmodel_L,ehats,sigs,ints,n,Jraw,z,dz,pz);     %
                [opt_val_L,x2_L,fvl2_L,exitflag2_L,output2_L,lambda2_L,grad2_L,hessian2_L]...           %
                    = M_step(theta0_L,nonlcon_samp_L,lx,ux,-p,options_fmincon);                         %
            elseif eam_U_conv == 0 && eam_L_conv == 1                                                   % Case 3: UB not yet, LP converged
                theta0_U = set_initial_value(x2_U,x2_U,exitflag2_U);                             		%
                nonlcon_samp_U = @(theta)nonl_const(theta,dmodel_U,ehats,sigs,ints,n,Jraw,z,dz,pz);     %
                [opt_val_U,x2_U,fvl2_U,exitflag2_U,output2_U,lambda2_U,grad2_U,hessian2_U]...           %
                    = M_step(theta0_U,nonlcon_samp_U,lx,ux,p,options_fmincon);                          %
            end
        end
        tm(iter)=toc;
        
        flag_temp_U(iter)=exitflag2_U;		% store exitflag (UB)
        flag_temp_L(iter)=exitflag2_L;		% store exitflag (LB)
        theta_U_iter(iter,1) = opt_val_U;   % store UB of CI
        theta_L_iter(iter,1) = opt_val_L;	% store LB of CI
        
        % Draw additional evaluation points
        [eam_U_conv, Theta_U_add]  = add_points(iter,iter_min,opt_val_U,theta_U_iter,...
            Theta_U_add,exitflag2_U,outer_tol,h_init,madd,d,lx,ux,p);
        [eam_L_conv, Theta_L_add]  = add_points(iter,iter_min,opt_val_L,theta_L_iter,...
            Theta_L_add,exitflag2_L,outer_tol,h_init,madd,d,lx,ux,-p);
		
		% If UB and LB both converge, exit the loop 
        if eam_U_conv == 1 && eam_L_conv == 1 
            break;
        end
        
		% If UB or LB cannot be found, exit the loop
        if iter > iter_min  && (max(flag_temp_U(iter-iter_min:iter)) <=0 || max(flag_temp_L(iter-iter_min:iter))<=0 )
            break;
        end
    end
    % Store UB and LB of CI
    if flag_temp_U(iter) > 0 && flag_temp_L(iter) > 0
        that_U(s) = theta_U_iter(iter);
        that_L(s) = theta_L_iter(iter);
        x_that_U(s,:) = x2_U';
        x_that_L(s,:) = x2_L';
    else
        that_U(s) = -Inf;
        that_L(s) = Inf;
        x_that_U(s,:) = NaN(1,d);
        x_that_L(s,:) = NaN(1,d);
    end
    steps(s) = iter;                            % # of EAM iterations
    Neval(s,:) = [length(Theta_L_current)...    % # of evaluation points
        length(Theta_U_current)];
    if ~isequal(x1_U,x1_L)
        c_at_xu(s) = C_U_current(1);
        c_at_xl(s) = C_L_current(2);
    else
        c_at_xu(s) = C_U_current(1);
        c_at_xl(s) = C_L_current(1);            % if x1_U=x1_L, we drop x1_L from initial eval pts
    end
    phi_at_xu(:,s) = get_gms(x1_U,Jraw,z,dz,pz,e_ineq,s_ineq,ki,n);
    phi_at_xl(:,s) = get_gms(x1_L,Jraw,z,dz,pz,e_ineq,s_ineq,ki,n);
    optim_val_U(:,s) = theta_U_iter;            % store UB of CI
    optim_val_L(:,s) = theta_L_iter;            % store LB of CI
    optim_flag_U(:,s) = flag_temp_U;            % store exitflags (UB)
    optim_flag_L(:,s) = flag_temp_L;            % store exitflags (LB)
    e_step_time(s) = sum(te);                   % store total time for E-Step
    a_step_time(s) = sum(ta);                   % store total time for A-Step
    m_step_time(s) = sum(tm);                   % store total time for M-Step
    disp(['simulation ' num2str(s) ' done.'])
end

% Store and report results
coverage_U = sum((theta_U<=that_U).*(theta_U >= that_L))/Nmc;
coverage_L = sum((theta_L<=that_U).*(theta_L >= that_L))/Nmc;
med_ci = [median(that_L) median(that_U)];
ci_length = median((that_U-that_L)-(theta_U-theta_L));
c_time = median(e_step_time./(sum(Neval,2)-minit));
disp(['True Projection = [' num2str(theta_L) ', ' num2str(theta_U) ']'])
disp(['Median CI       = [' num2str(med_ci(1)) ', ' num2str(med_ci(2)) ']'])
disp(['Coverage Probability of CI for theta_U = ' num2str(coverage_U)])
disp(['Coverage Probability of CI for theta_L = ' num2str(coverage_L)])
disp(['Excess Length of CI = ' num2str(ci_length)])
disp(['Time needed to compute cv is ' num2str(c_time) ' seconds.'])
disp(['Time spent for EAM steps are '])
disp(['E-Step: ' num2str(median(e_step_time)) ' seconds'])
disp(['A-Step: ' num2str(median(a_step_time)) ' seconds'])
disp(['M-Step: ' num2str(median(m_step_time)) ' seconds.'])
savefile = ['./Results/8D' method '_n' num2str(n) '_alpha' num2str(alpha)...
    '_component' num2str(component) '_' id 'ID.mat'];
if ~exist('./Results','dir')
    mkdir('./Results');
end
save(savefile, ... 
    'alpha', 'Beta', 'theta_true','p', 'selp', 'z', 'theta_U','theta_L',...
    'Nmc', 'n', 'M', 'siglb', 'x1*','rho','ki','id',...
    'mbase', 'dace*', 'niter',...
    'C_*_current',...
    'optim_flag*', 'optim_val*',...
    'that*','x_that*',...
    '*step_time','c_time',...
    'coverage*','med_ci','ci_length'); 
disp('done')
end

function [z,x,dz,pz,theta_true,lx,ux,p] = set_specification(component,id)
% This function sets parameters in each DGP
% INPUT
% component: coordinate of interest
% OUTPUTS
% z: support of covariates (X and W)
% x: support of X
% theta_true: true value of theta
% lx, ux: upper and lower bounds of parameter space
% p: direction of projection
x = [1 -1 1 -1
    1 -1 1 1
    1 1  1 -1
    1 1  1 1 ];
z = [x x];
pz = [0.1;0.2;0.3;0.4];
dx1 = 2; dx2 = 2; dw1 = 2; dw2 = 2;
b1 = 0.5; b2 = 0.25;
d1 = -1;
if strcmp(id,'point')
    d2 = -1;
elseif strcmp(id,'partial')
    d2 = -0.75;
end
beta1_true = [b1;b2];
beta2_true = [b1;b2];
delta1_true = [d1;d2];
delta2_true = [d1;d2];
lx = [-2*ones(1,4) -4*ones(1,4)];
ux = [2*ones(1,4) zeros(1,4)];
p   = zeros(8,1);
p(component) = 1;
dz = [dx1;dx2;dw1;dw2];
theta_true = [beta1_true
    beta2_true
    delta1_true
    delta2_true];
end

function rho = get_rho(Beta,J,d)
% This function computes rho from Beta
% INPUTS
% Beta: amount of conservativeness
% OUTPUT
% rho

rho  = -norminv((1-power(1-Beta,1/(d*nchoosek(J,d))))/2);
end

function [x1,varargout] = population_proj(theta0,p,lx,ux,nonlcon_pop,options_fmincon)
% This function computes the population projection of the identified set
% INPUTS
% theta0: initial value for optimization
% p: direction of projection
% lx, ux: lower and upper bounds on the parameter space
% nonlcon_pop: nonlinear constraints that define the identified set
% options_fmincon: options to be passed to fmincon
% x1: argmin of the problem
[x1,fvl1,exitflag1,output1,lambda1,grad1,hessian1]  = fmincon(@(theta)linobj(theta,p),theta0,...
    [],[],[],[],lx,ux,nonlcon_pop,...
    options_fmincon);
if exitflag1 <= 0 
    ms = MultiStart;
    ms.StartPointsToRun = 'all';
    problem = createOptimProblem('fmincon','x0',theta0,...
    'objective',@(theta)linobj(theta,p),'lb',lx,'ub',ux,...
    'nonlcon',nonlcon_pop,...
    'options',options_fmincon);
    [x1,fvl1,exitflag1,lambda1,manyminsm] = run(ms,problem,50);
end

varargout{1} = fvl1;
varargout{2} = exitflag1;
varargout{3} = output1;
varargout{4} = lambda1;
varargout{5} = grad1;
varargout{6} = hessian1;
end

function [momeq, momineq,varargout] = population_freq(theta,z,dz,pz,Jraw,selp)
% This function computes population moments (pmom) if selp is nonempty
% If selp is nonempty, it computes the theoretical bounds (bound).
% The function assumes the following form:
% pmomeq - boundeq = 0
% pmomineq - boundeq <=0
% INPUTS
% theta: true parameter
% z: Nz-by-length(dz), support of X and W
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% Jraw: # of (raw) inequalities
% selp: selection probability
% OUTPUTS
% momeq: 1st term of population mom equality if selp is nonempty
%        2nd term (bound) of population mom equality if selp is empty
% momineq: similar to momeq (counterpart for inequality)
% varargout include indicators that show whether the frequency is in interior of [0,1]

int_threshold = 5/3000;
dx1 = dz(1);
dx2 = dz(2);
dw1 = dz(3);
dw2 = dz(4);
beta1  = theta(1:dx1);
beta2  = theta(dx1+1:dx1+dx2);
delta1 = theta(dx1+dx2+1:dx1+dx2+dw1);
delta2 = theta(dx1+dx2+dw1+1:end);

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
momeq = zeros(Jeq,1);
momineq = zeros(Jineq,1);

for l = 1:Nz
    zval = z(l,:);
    x1 = zval(1:dx1);
    x2 = zval(dx1+1:dx1+dx2);
    w1 = zval(dx1+dx2+1:dx1+dx2+dw1);
    w2 = zval(dx1+dx2+dw1+1:end);
    momeq((l-1)*Jeqraw+1,1)   = normcdf(-x1*beta1)...
        *normcdf(-x2*beta2)*pz(l);
    momeq((l-1)*Jeqraw+2,1)   = (1 - normcdf(-x1*beta1-w1*delta1))...
        *(1-normcdf(-x2*beta2-w2*delta2))*pz(l);
    momineq((l-1)*Jineqraw+1,1) = normcdf(-x1*beta1-w1*delta1)...
        *(1-normcdf(-x2*beta2))*pz(l);
    momineq((l-1)*Jineqraw+2,1) = -(normcdf(-x1*beta1-w1*delta1)...
        *(1-normcdf(-x2*beta2-w2*delta2))...
        +normcdf(-x1*beta1)...
        *(normcdf(-x2*beta2-w2*delta2)-normcdf(-x2*beta2)))*pz(l);
    interior_eq = (abs(momeq)>int_threshold).*(abs(momeq) < (1-int_threshold));
    interior_ineq = (abs(momineq)>int_threshold).*(abs(momineq) < (1-int_threshold));
    varargout{1} = interior_eq;
    varargout{2} = interior_ineq;
    if ~isempty(selp)
        width = momineq((l-1)*Jineqraw+1,1) + momineq((l-1)*Jineqraw+2,1);
        momineq((l-1)*Jineqraw+1,1) = momineq((l-1)*Jineqraw+1,1) - width*selp;
        momineq((l-1)*Jineqraw+2,1) = -momineq((l-1)*Jineqraw+1,1);
    end
end
end


function [c,ceq,GC,GCeq] = population_nonlcon(theta,theta_true,z,dz,pz,Jraw,selp)
% This function computes population moment constraints
% INPUTS
% theta: true parameter
% z: Nz-by-length(dz), support of X and W
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% Jraw: # of (raw) inequalities
% selp: selection probability
% OUTPUTS
% c, GC: equality constraints and their gradients
% ceq, GCeq: inequality ocnstraints and their gradients

eq_tol = 1;
d = length(theta);
Nz = size(z,1);
Jeq = Jraw(1)*Nz;
[pmomeq,pmomineq,pinteq,pintineq] = population_freq(theta_true,z,dz,pz,Jraw,selp);
sigeq   = (sqrt(pmomeq.*(1-pmomeq)));
sigineq = sqrt(abs(pmomineq).*(1-abs(pmomineq)));
[boundeq,boundineq] = population_freq(theta,z,dz,pz,Jraw,[]);
[Greq, Grineq] = get_gradient(theta,Jraw,z,dz,pz);
GC = eq_tol.*(Grineq./repmat(sigineq,1,d).*repmat(pintineq,1,d))';
GCeq = eq_tol.*(Greq(1:Jeq,:)./repmat(sigeq,1,d).*repmat(pinteq,1,d))';

ceq = eq_tol.*((pmomeq - boundeq)./sigeq).*pinteq;
c = eq_tol.*((pmomineq - boundineq)./sigineq).*pintineq;
end


function [X1,X2,U1,U2,V] = drawsample(n,Nmc,selp,z,dz,pz)
% This function draws all samples across markets and MC replications
% INPUTS
% n: sample size
% Nmc: # of MC replications
% selp: selection probability
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% OUTPUTS
% X1,X2: n x Nmc binary covariates
% U1,U2: n x Nmc latent variables
% V: n x Nmc latent binary variable (=1 w.p. selp)

dx1 = dz(1);
dx2 = dz(2);

U1 = randn(n,Nmc);
U2 = randn(n,Nmc);
V  = (rand(n,Nmc)>selp);
temp = gendist(pz',n,Nmc);
X1V = get_nonconstantx(z(temp(:),1:dx1));
X2V = get_nonconstantx(z(temp(:),dx1+1:dx1+dx2));
X1 = cat(2,ones(n,1,Nmc),reshape(X1V,n,dx1-1,Nmc));
X2 = cat(2,ones(n,1,Nmc),reshape(X2V,n,dx2-1,Nmc));
end


function [y] = outcome(xb1,xb2,wd1,wd2,u1,u2,v)
% This function generates outcomes for each market
% INPUTS
% xb1,xb2: scalars X1'beta1 and X2'beta2
% wd1,wd2: scalars W1'delta1 and W2'delta2 
% u1,u2: scalar u1 and u2
% v: binary variable that determines selection
%
% OUTPUTS
% y: outcome, 0, 11, 1, or 10

y=NaN;
    % (1) unique equlibrium (0,0)
if (u1 < -xb1 && u2 < -xb2)
    y = 0;
    % (2) unique equlibirum (1,1)
elseif (u1 > -xb1-wd1 && u2 > -xb2-wd2)
    y = 11;
    
    % (3) unique eqilibirum (0,1)
elseif ( u1<-xb1-wd1 && u2>-xb2-wd2 )...
        || ( u1<-xb1 &&  u2<-xb2-wd2...
        && u2>-xb2  )
    y = 01;
    % (4) multiple equlibrium region, select (0,1) with probability
    % (1-selp)
elseif (u1>-xb1 && u1<-xb1-wd1...
        && u2>-xb2 && u2<-xb2-wd2)
    if v==1
        y = 10;
    elseif v==0
        y = 01;
    end
else % unique eqlibirum  (1 0)
    y= 10;
end
end

function XV = get_nonconstantx(X)
% This function extracts variables in X that are not constant.
% INPUT
% X: n x dx x Nmc array
% OUTPUT
% XV: n x (dxv*Nmc) matrix, where dxv is the number of nonconstant variables

[n,dx,Nmc] = size(X);
temp = reshape(X,n,dx*Nmc);
XV = temp(:,~ismember(temp',ones(1,n),'rows'));
end

function phat = emp_freq(Y,X1V,X2V,Jraw,x)
% This function coputes empirical frequencies
% INPUTS
% X1V,X2V: n x Nmc binary covariates
% x: Nz-by-2, support of X
% Jraw: # of (raw) inequalities
% OUTPUT
% phat: (Jeq + Jineq) x Nmc empirical frequencies
% Note: components corresponding to the 2nd raw mom ineq take negative
% values

[n,Nmc] = size(Y);
Nz = size(x,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
phateq = zeros(Jeq,Nmc);
phatineq = zeros(Jineq,Nmc);

if isempty(X1V) && isempty(X2V)
    phateq(1,:)   = sum(Y==0)/n;
    phateq(2,:)   = sum(Y==11)/n;
    phatineq(1,:) = sum(Y==1)/n;
    phatineq(2,:) = -sum(Y==1)/n;
else
xv=x(:,~ismember(x',ones(1,Nz),'rows'));
for l = 1:Nz
    zval = xv(l,:);
    x1v = zval(1);
    x2v = zval(2);
    phateq((l-1)*Jeqraw+1,:)   = sum(Y==0  & X1V == x1v & X2V == x2v)/n;
    phateq((l-1)*Jeqraw+2,:)   = sum(Y==11 & X1V == x1v & X2V == x2v)/n;
    phatineq((l-1)*Jineqraw+1,:) = sum(Y==1  & X1V == x1v & X2V == x2v)/n;
    phatineq((l-1)*Jineqraw+2,:) = -sum(Y==1 & X1V == x1v & X2V == x2v)/n;
end
end

phat = [phateq; phatineq];
end


function [sigeq,sigineq] = get_sigma(phateq,phatineq)
% This function coputes estimates of standard deviations
% INPUTS
% phateq:   Jeq x Nmc empirical frequencies for equalities
% phatineq: Jineq x Nmc empirical frequencies for inequalities
% OUTPUT
% sigeq:   2*Jeq x Nmc std devs for equalities
% sigineq: Jineq x Nmc std devs for inequalities

sigeq   = (sqrt(phateq.*(1-phateq)));
sigeq   = [sigeq; sigeq];
sigineq = sqrt(abs(phatineq).*(1-abs(phatineq)));
end

function[bound_eq,bound_ineq] = get_bound(theta,Jraw,z,dz,pz)
% This function computes the 2nd term in moment (in)equalities.
% INPUTS
% theta: d x 1 parameter value
% Jraw: # of (raw) inequalities
% z: Nz-by-2, support of X
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% OUTPUTS
% bound_eq: 2*Jeq x 1 bounds for equalities
% bound_ineq: Jineq x 1 bounds for inequalities

Nz = size(z,1);
Jeqraw = Jraw(1);
Jeq = Jeqraw * Nz;

bound_eq = zeros(Jeq*2,1);
[temp1,temp2] = population_freq(theta,z,dz,pz,Jraw,[]);
bound_eq(1:Jeq,:) =  temp1;
bound_eq(Jeq+1:end,:) = - temp1;
bound_ineq = temp2;
end


function [momeq,momineq,varargout] = get_moment(phateq,phatineq)
% This function coputes estimates of 1st term of sample moments
% INPUTS
% phateq:   Jeq x Nmc empirical frequencies for equalities
% phatineq: Jineq x Nmc empirical frequencies for inequalities
% OUTPUT
% momeq:   2*Jeq x Nmc sample moments for equalities
% momineq: Jineq x Nmc sample moments for inequalities
% varargout: contains indicators for moments being away the boundary

int_treshold = 1e-4;
[Jeq,Nmc] = size(phateq);
momeq = zeros(Jeq*2,Nmc);

momeq(1:Jeq,:) = phateq;
momeq(Jeq+1:end,:) = -phateq;
momineq  = phatineq;

interior_eq = (abs(momeq)>int_treshold).*(abs(momeq) < (1-int_treshold));
interior_ineq = (abs(momineq)>int_treshold).*(abs(momineq) < (1-int_treshold));
varargout{1} = interior_eq;
varargout{2} = interior_ineq;
end

function [Geq, Gineq] = get_gradient(theta,Jraw,z,dz,pz)
% This function coputes gradients (not-studentized) of moments 
% INPUTS
% theta: true parameter
% z: Nz-by-2, support of Z
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% pz: distribution of Z
% Jraw: # of (raw) inequalities
% OUTPUT
% Geq:   2*Jeq x d gradient for equalities
% Gineq: Jineq x d gradient for inequalities

d = length(theta);
dx1 = dz(1);
dx2 = dz(2);
dw1 = dz(3);
dw2 = dz(4);
beta1  = theta(1:dx1);
beta2  = theta(dx1+1:dx1+dx2);
delta1 = theta(dx1+dx2+1:dx1+dx2+dw1);
delta2 = theta(dx1+dx2+dw1+1:end);

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
Geq = zeros(Jeq,d);
Gineq = zeros(Jineq,d);

for l = 1:Nz
   zval = z(l,:);
   x1 = zval(1:dx1);
   x2 = zval(dx1+1:dx1+dx2);
   w1 = zval(dx1+dx2+1:dx1+dx2+dw1);
   w2 = zval(dx1+dx2+dw1+1:end);
   G1b1 = x1.*normpdf(-x1*beta1).*normcdf(-x2*beta2)*pz(l);
   G1b2 = x2.*normcdf(-x1*beta1).*normpdf(-x2*beta2)*pz(l);
   G1d1 = zeros(1,dw1);
   G1d2 = zeros(1,dw2);
   
   G2b1 = -x1.*normpdf(-x1*beta1-w1*delta1).*(1-normcdf(-x2*beta2-w2*delta2))*pz(l);
   G2b2 = -x2.*(1-normcdf(-x1*beta1-w1*delta1)).*normpdf(-x2*beta2-w2*delta2)*pz(l);
   G2d1 = -w1.*normpdf(-x1*beta1-w1*delta1).*(1-normcdf(-x2*beta2-w2*delta2))*pz(l);
   G2d2 = -w2.*(1-normcdf(-x1*beta1-w1*delta1)).*normpdf(-x2*beta2-w2*delta2)*pz(l);
   
   G3b1 = x1.*normpdf(-x1*beta1-w1*delta1).*(1-normcdf(-x2*beta2))*pz(l);
   G3b2 = -x2.*normcdf(-x1*beta1-w1*delta1).*normpdf(-x2*beta2)*pz(l);
   G3d1 = w1.*normpdf(-x1*beta1-w1*delta1).*(1-normcdf(-x2*beta2))*pz(l);
   G3d2 = zeros(1,dw2);
   
   G4b1 = -x1.*normpdf(-x1*beta1-w1*delta1).*(1-normcdf(-x2*beta2-w2*delta2))*pz(l) ...
       -x1.*normpdf(-x1*beta1).*(normcdf(-x2*beta2-w2*delta2)-normcdf(-x2*beta2))*pz(l);
   G4b2 = x2.*normcdf(-x1*beta1-w1*delta1).*normpdf(-x2*beta2-w2*delta2)*pz(l) ...
       +normcdf(-x1*beta1).*(-x2.*normpdf(-x2*beta2-w2*delta2)+x2.*normpdf(-x2*beta2))*pz(l);
   G4d1 = -w1.*normpdf(-x1*beta1-w1*delta1).*(1-normcdf(-x2*beta2-w2*delta2))*pz(l);
   G4d2 = w2.*normcdf(-x1*beta1-w1*delta1).*normpdf(-x2*beta2-w2*delta2)*pz(l) ...
       +w2.*normcdf(-x1*beta1).*(-normpdf(-x2*beta2-w2*delta2))*pz(l);
    
   Geq((l-1)*Jeqraw+1,:) = [G1b1 G1b2 G1d1 G1d2];
   Geq((l-1)*Jeqraw+2,:) = [G2b1 G2b2 G2d1 G2d2];
   Gineq((l-1)*Jineqraw+1,:) = [G3b1 G3b2 G3d1 G3d2];
   Gineq((l-1)*Jineqraw+2,:) = [G4b1 G4b2 G4d1 G4d2];
end
Geq = [Geq; -Geq];
end

function C_temp = E_step(Theta,n,Jraw,z,dz,pz,Gs,e_eq,...
    e_ineq,s_eq,s_ineq,pint_eq,pint_ineq,ki,alpha,rho,p,type,method)
% This function computes critical values over evaluation points 
% Key INPUTS	
% Theta: m-by-d evaluation points
% method: 'KMS' or 'AS'
% OUTPUT
% C_temp: m-by-1 critical values over evaluatoin points

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
[m,d] = size(Theta);
C_temp = zeros(m,1);
parfor k = 1:m % loop over evaluation points
    theta = Theta(k,:)';
    
    % Compute objects that require theta
    phis = get_gms(theta,Jraw,z,dz,pz,e_ineq,s_ineq,ki,n);
    [Greq, Grineq] = get_gradient(theta,Jraw,z,dz,pz);            % compute gradient at theta
    Ds = [Greq./repmat(s_eq,1,d).*repmat(pint_eq,1,d);            % compute D at theta
        Grineq./repmat(s_ineq,1,d).*repmat(pint_ineq,1,d)];  
    Ds = replace_moments(Ds,phis,Jeq,Jineqraw,Nz);				  % merge paiered inequalities
    if strcmp(method,'KMS')
    C_B =  get_Bonferroni(Gs,alpha);                              % get a Bonferroni cv bound
        C_temp(k) = C_KMS_lin(Gs,Ds,phis,C_B,alpha,rho,p,type);	  % get a KMS cv
    elseif strcmp(method,'AS')
        C_temp(k) = C_AS_lin(Gs,phis,alpha);					  % get an AS cv
    end            
end
end

function dmodel = A_step(Theta_current,C_current,dace_theta,dace_lob,dace_upb)
% This function fits a surrogate model
% INPUTS
% Theta_current: evaluation points
% C_current: critical values
% dace*: parameters used by dacefit
% OUTPUT
% dmodel: fitted model 

[dmodel, ~] = dacefit(Theta_current,C_current,@regpoly0,@corrgauss,...      
    dace_theta,dace_lob,dace_upb);
end

function phis = get_gms(theta,Jraw,z,dz,pz,e_ineq,s_ineq,ki,n)
% This function calculates GMS
% INPUTS
% theta: parameter value
% Jraw: # of original constraints
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% pz: distribution of covariates
% e_ineq: empirical frequencies associated with inequalities
% s_ineq: estimated sigma associated with inequalities
% ki: kappa (GMS tuning parameter)
% n: sample size
% OUTPUT
% phis: GMS term
	
Nz = size(z,1);
Jeqraw = Jraw(1);
Jeq = Jeqraw * Nz;
[~,bound_ineq] = get_bound(theta,Jraw,z,dz,pz);
sqnmom_ineq = sqrt(n).*(e_ineq - bound_ineq)./s_ineq;
phis_ineq = double((sqnmom_ineq/ki<-1));                                
phis = [zeros(2*Jeq,1);phis_ineq];
end

function Ds = replace_moments(Ds,phis,Jeq,Jineqraw,Nz)
% This function merges paiered inequalities if both bind.
% Key INPUTS
% Ds: Gradients of constraints
% phis: GMS term
% OUTPUT
% Ds: "merged" gradients

Jineq_start = 2*Jeq;
for l = 1:Nz
    if phis(Jineq_start+(l-1)*Jineqraw+1) == 0 && ...
            phis(Jineq_start+(l-1)*Jineqraw+2) == 0
        Ds(Jineq_start+(l-1)*Jineqraw+2) = -Ds(Jineq_start+(l-1)*Jineqraw+1);
    end
end
end

function theta0 = set_initial_value(theta_true,x,exitflag)
% This function sets the initial value of optimization
if exitflag > 0
    theta0 = x;                                                         %
else
    theta0 = theta_true;
end
end

function [opt_val,varargout] = M_step(theta0,nonlcon_samp,lx,ux,p,options_fmincon)
% This function solves an optimization problem to obtain the end point of CI
% INPUTS
% theta0: initial value for optimization
% nonlcon_samp: sample nonlinear constraints
% lx, ux: bounadry of the parameter space
% p: projection direction
% options_fmincon: options passed to fmincon
% opt_val: optimal value

[x2,fvl2,exitflag2,output2,lambda2,grad2,hessian2]...
    = fmincon(@(theta)linobj(theta,p),theta0,[],[],[],[],lx,ux,...
    nonlcon_samp,options_fmincon);

if exitflag2 <=0 
    ms = MultiStart;
    ms.StartPointsToRun = 'all';
    problem = createOptimProblem('fmincon','x0',theta0,...
    'objective',@(theta)linobj(theta,p),'lb',lx,'ub',ux,...
    'nonlcon',nonlcon_samp,...
    'options',options_fmincon);
    [x2,fvl2,exitflag2,lambda2,manyminsm] = run(ms,problem,50);
end

opt_val = x2(p~=0);
varargout{1} = x2;
varargout{2} = fvl2;
varargout{3} = exitflag2;
varargout{4} = output2;
varargout{5} = lambda2;
varargout{6} = grad2;
varargout{7} = hessian2;
end

function C_B =  get_Bonferroni(Gs,alpha)
% This function computes a Bonferroni cv
C_B = quantile(max(Gs,[],2),1-alpha);
end


function C_KMS = C_KMS_lin(Gs,Ds,phis,C_B,alpha,rho,p,type)            
% This function solves the inner loop problem and calibrates c for a given
% theta using KMS critical value                                                        
% IMPUTS                                                                       
% Gs: bootstraped moments (centered)                                     
% Ds: gradients                                  
% phis: GMS term                                 
% C_B: initial upper bound on C_KMS (e.g. Bonferroni)                                                                        
% alpha: nominal level
% rho: rho-box's size
% p: projection direction
% type: 'two-sided' or 'one-sided'                                               

M = size(Gs,2);			                                                    % # of bootstrap replications
kmax=12;                                                                    % maximum number of bisecions
TEMP = [zeros(M,1),ones(M,1)];                                              % TEMP(:,1), TEMP(:,2) hold bootstrap replications covered under C_L & C_U respectively.
C_STORE = [NaN(1,kmax), 0, C_B];                                            % vector storing coverage probs in LPs
[coverage_0,TEMP_0] = get_coverage_LP(Gs,Ds,phis,0,TEMP,rho,p,type);	    % coverage probability in LP when C=0 
[coverage_B,TEMP_B] = get_coverage_LP(Gs,Ds,phis,C_B,TEMP,rho,p,type);      % coverage probability in LP when C=C_B 
if coverage_B < 1-alpha                                                     % 
    C_B = 2*C_B;                                                            % if C_B is too low, scale up the upper bound
    [coverage_B,TEMP_B] = get_coverage_LP(Gs,Ds,phis,C_B,TEMP,rho,p,type);  %
end                                                                         %
COVER_STORE = [NaN(1,kmax) coverage_0 coverage_B];                          %
TEMP = [TEMP_0,TEMP_B];                                                     % TEMP (M-by-2) stores replications covered under C=0 and C=C_B.
                                                                               
if coverage_0 >= 1-alpha;                                                   % if C=0 does the job,
    C_KMS = 0;                                                              % set C_KMS=0 and exit
else                                                                        % else, start the algorithm
    CL = 0;                                                   			    % 
    CU = C_B;                                                               % set an initial C as the midpoint of [0,C_B]
    C = (CL+CU)/2;                                                        	
    for k = 1:kmax;                                                           
        [coverage,T] = get_coverage_LP(Gs,Ds,phis,C,TEMP,rho,p,type);       % compute coverage at C
        COVER_STORE(k) = coverage;                                             
        C_STORE(k) = C;                                                        
        alpha_hat = 1-coverage;                                                
        if abs(coverage - (1-alpha)) <= 1/M && coverage >= 1-alpha          % if coverage at current C is close enough to 1-alpha
            C_KMS = C;                                                      % set C_KMS to C and exit
            break;                                                          %
        else                                                                %
            if alpha_hat>alpha;                                             % alpha_hat>alpha means coverage is less than desired, hence
                CL = C;                                                     % expand c: replace lower limit with C and search between C and CU
                C = (CL+CU)/2;                                              %
                TEMP(:,1) = T(:,1);                                         %
            else                                                            % If coverage is more than desired, then contract: restirct your
                CU = C;                                                     % search from CL to C
                C = (CL+CU)/2;                                              %
                TEMP(:,2) = T(:,1);                                         %
            end
        end%
    end
    
    if k==kmax                                                              % if the bisection search continued until kmax is reached,
        cover_diff = COVER_STORE - (1-alpha);                               % choose C that gave the coverage probability closest to 1-alpha (from above).
        rpos = find(cover_diff>0);                                          %
        [~,U_min] = min(C_STORE(rpos));                                     %
        r_U = rpos(U_min);                                                  %
        C_KMS = C_STORE(r_U);                                               %
    end                                                                     %
end
end

function [coverage, T] = get_coverage_LP(Gs,Ds,phis,C,TEMP,rho,p,type)
% This function solves the inner loop problem for a given c and theta      
% INPUTS
% Gs: bootstraped moments (centered)                                     
% Ds: gradients                                  
% phis: GMS term   
% C: current amount of expansion (between C_L and C_U)
% TEMP: matrix (Mx2) tracking coverage across bootstrap replications at C_L and C_U
% rho: rho-box's size
% p: projection direction
% type: 'two-sided' or 'one-sded'
% OUTPUTS
% coverage: coverage probability of LP
% T: M-by-1 vector indicating bootstrap replications "covered" under C           

params=struct('A',{},'b',{},'p',{});                                      
settings.verbose = 0;                                                     
settings.eps = 1e-9;                                                      
Jbind = sum(phis==0);   
[Jall,M] = size(Gs);
d = size(Ds,2);
K = [1:1:M]';                                                             
idx_u = K((TEMP(:,2) - TEMP(:,1))==1);                                                          
mu = length(idx_u);                                                       
that1_boot = zeros(mu,1);                                                            
that2_boot = zeros(mu,1);                                                                             
nuhat_boot = zeros(mu,1);                                                                                    
boot_GS_u = Gs(:,idx_u);                                                  
for j = 1:mu                                                              
    W    = -boot_GS_u(:,j);
    if strcmp(type,'two-sided')
        params(j).A = [Ds(phis==0,:)
            eye(d)
            -eye(d)
            zeros(Jall-Jbind,d)
            -p'
            p'];
        params(j).b = [W((phis==0))+C
            rho*ones(2*d,1)
            ones(Jall-Jbind,1)
            0
            0];
    elseif strcmp(type,'one-sided')
        params(j).A = [Ds(phis==0,:)
            eye(d)
            -eye(d)
            zeros(Jall-Jbind,d)
            -p'
            zeros(1,d)];
        params(j).b = [W((phis==0))+C
            rho*ones(2*d,1)
            ones(Jall-Jbind,1)
            0
            0];
    end
    
    switch d %
        case 4
        [vars,status] = csolve4(params(j),settings);                            
        case 6
        [vars,status] = csolve6(params(j),settings);
        case 8
        [vars,status] = csolve8(params(j),settings); % Check feasibility using CVXGEN
    end
    
    if status.converged==1                  % If the set is feasible, store results                              
        that1_boot(j) = vars.lambda(1);                                   
        that2_boot(j) = vars.lambda(2);                                   
        nuhat_boot(j) = status.optval;                                    
    else                                    % otherwise, store -Inf                              
        that1_boot(j) = -Inf;                                             
        that2_boot(j) = -Inf;                                             
        nuhat_boot(j) = -Inf;                                             
    end                                                                   
end                                                                       
T_u = zeros(M,1);                                                         
T_u((TEMP(:,2) + TEMP(:,1))==2) = 1;                                      
T_u(idx_u) = (nuhat_boot>=0);                                             
coverage = sum((T_u==1))/M;
T = T_u;
end


function C = C_AS_lin(Gs,phis,alpha)
% This function computes AS's critical value
% INPUTS
% Gs: bootstrapped moments
% phis: GMS
% alpha: nominal level	
	
[J,M] = size(Gs);
phi=zeros(J,M);
phi(phis==1,:)=-Inf;
S_3_boot = max(Gs+phi);
C = quantile(S_3_boot,1-alpha);                                      
end



function [c,ceq,GC,GCeq] = nonl_const(theta,dmodel,ehats,sigs,ints,n,Jraw,z,dz,pz)
% This function computes the nonlinear constraint for the projection
% problem
% INPUTS
% theta:  parameter
% dmodel: surrogate model (output from dace_fit function) for cv
% ehats:  J x 1, 1st term in the moment conditions
% sigs:   J x 1, std devs
% ints:   J x 1, indicator for moments being away from boundary
% n:      scalar, sample size
% z:      Nz-by-length(dz), support of X and W
% dz=[dx1; dx2; dw1; dw2]: dimensions of covariates
% Jraw:   # of (raw) inequalities
% OUTPUT
% c:      inequality constraint
% GC:     gradient

d = size(theta,1);
ceq = 0;
GCeq = zeros(d,1);

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
J = 2*Jeq + Jineq;

[bound_eq,bound_ineq] = get_bound(theta,Jraw,z,dz,pz);
[Greq, Grineq] = get_gradient(theta,Jraw,z,dz,pz);
Gr = [Greq; Grineq];
Ds = Gr./repmat(sigs,1,d);
bound = [bound_eq;bound_ineq];
moms = (ehats-bound)./sigs;
[chat,cgrad] = predictor(theta,dmodel);
c = (sqrt(n)*moms-repmat(chat,J,1)).*ints;
GC = sqrt(n)*Ds.*repmat(ints,1,d) - repmat(cgrad',J,1);
GC = GC';
end

function [f,g] = linobj(theta,p)
% This function computes the objective function for the projection
% problem
% INPUTS
% theta: parameter
% p:     direction of interest
% OUTPUT
% f:     function value
% g:     gradient

f = -p'*theta;

if nargout > 1
    g = -p;
end
end

function [eam_conv, Theta_add]  = add_points(iter,iter_min,opt_val,theta_iter,...
    Theta_add,exitflag2,outer_tol,h_init,madd,d,lx,ux,p)
% This function add evaluation points until optimal values converge
% INPUTS
% iter: current iteration of EAM algorithm
% iter_min: minimum # of iterations
% opt_val: optimal value from the latest M-step
% theta_iter: history of optimal values across iterations
% Theta_add: matrix storing additional evaluation points
% exitflag2: exitflag from latest M-step
% outer_tol: tolerance for exiting the iteration of EAM steps
% h_init: initial width of the set to which evaluation points are drawn
% m_add: # of additional evaluation points
% lx,ux: lower and upper bounds of parameter space
% p: projection direction
% OUTPUTS
% eam_conv: =1 if EAM algorithm converged, =0 otherwise
% Theta_add: updated matrix storing additional evaluation points
	
if iter>iter_min && abs(theta_iter(iter)-theta_iter(iter-1)) < outer_tol && exitflag2>0
    eam_conv = 1; % if the current optimal- previous optimal is within a tolerance, break
else
    if exitflag2 >0
        h = h_init/iter^(1.5);   % otherwise, set bandwidth
        temp = draw_points(madd,d,lx,ux,h,p,opt_val);
    elseif exitflag2 <= 0
        temp = draw_points(madd,d,lx,ux);
    end
    Theta_add(madd*(iter-1)+1:madd*iter,:) = temp;
    eam_conv = 0;
end
end

function points = draw_points(madd,d,lx,ux,varargin)
% This function draws points on parameter space
% INPUTS
% madd: # of (additional) evaluation points
% d: dimension of theta
% lx: 1xd lower bound on parameter space
% ux: 1xd upper bound on parameter space
% varargin: additional arguments. If provided, the function draws points that achieve values close to fvl2.
% OUTPUTS
% points: madd x d evaluation points
	
  if nargin > 4   % if h, p, fvl2 are passed from outside, draw locally
    h = varargin{1};
    p = varargin{2};
    opt_val = varargin{3};
    temp = lhsamp(madd,d);                                                    % draw points in a cube (Latin hyepercube sampling)
    pone = find(p~=0);                                                        %
    pzero = find(p==0);                                                       %
    temp(:,pone) = opt_val + h*(temp(:,pone)-mean(temp(:,pone)));             % make the coordinate of interest around the current optimal
    temp(:,pzero) = repmat(ux(pzero)-lx(pzero),madd,1).*temp(:,pzero)...      % rescale other coordinates
        +repmat(lx(pzero),madd,1);     
    temp(temp(:,pone) > ux(pone),pone)=ux(pone);
    temp(temp(:,pone) < lx(pone),pone)=lx(pone);
  elseif nargin == 4   % otherwise, draw globally
    temp = lhsamp(madd,d);                                                    % draw points on a cube (Latin hypercube sampling)
    temp = repmat((ux-lx),madd,1).*temp+repmat(lx,madd,1);					  % rescale to draw points on parameter space	
  end
points = temp;
end


function A = repelem(A, varargin)
  index = cell(1, nargin-1);
  for iDim = 1:nargin-1
    lens = varargin{iDim};
    if isscalar(lens)
      if (lens == 1)
        index{iDim} = ':';
        continue
      else
        lens = repmat(lens, 1, size(A, iDim));
      end
    end
    vals = 1:size(A, iDim);
    clens = cumsum(lens);
    index{iDim} = zeros(1, clens(end));
    index{iDim}([1 clens(1:end-1)+1]) = diff([0 vals]);
    index{iDim} = cumsum(index{iDim});
  end
  A = A(index{:});
end

function T = gendist(P,N,M,varargin)
%GENDIST - generate random numbers according to a discrete probability
%distribution.
%Tristan Ursell, (c) 2012
%
%T = gendist(P,N,M)
%T = gendist(P,N,M,'plot')
%
%The function gendist(P,N,M) takes in a positive vector P whose values
%form a discrete probability distribution for the indices of P. The
%function outputs an N x M matrix of integers corresponding to the indices
%of P chosen at random from the given underlying distribution.
%
%P will be normalized, if it is not normalized already. Both N and M must
%be greater than or equal to 1.  The optional parameter 'plot' creates a
%plot displaying the input distribution in red and the generated points as
%a blue histogram.
%
%Conceptual EXAMPLE:
%
%If P = [0.2 0.4 0.4] (note sum(P)=1), then T can only take on values of 1,
%2 or 3, corresponding to the possible indices of P.  If one calls 
%gendist(P,1,10), then on average the output T should contain two 1's, four
%2's and four 3's, in accordance with the values of P, and a possible 
%output might look like:
%
%T = gendist(P,1,10)
%T = 2 2 2 3 3 3 1 3 1 3
%
%If, for example, P is a probability distribution for position, with
%positions X = [5 10 15] (does not have to be linearly spaced), then the
%set of generated positions is X(T).
%
%EXAMPLE 1:
%
%P = rand(1,50);
%T = gendist(P,100,1000,'plot');
%
%EXAMPLE 2:
%
%X = -3:0.1:3;
%P = 1+sin(X);
%T = gendist(P,100,1000,'plot');
%Xrand = X(T);
%
%Note:
%size(T) = 100 1000
%size(Xrand) = 100 1000
%

%Thanks to Derek O'Connor for tips on speeding up the code.

%check for input errors
if and(nargin~=3,nargin~=4)
    error('Error:  Invalid number of input arguments.')
end

if min(P)<0
    error('Error:  All elements of first argument, P, must be positive.')
end

if or(N<1,M<1)
    error('Error:  Output matrix dimensions must be greater than or equal to one.')
end

%normalize P
Pnorm=[0 P]/sum(P);

%create cumlative distribution
Pcum=cumsum(Pnorm);

%create random matrix
N=round(N);
M=round(M);
R=rand(1,N*M);

%calculate T output matrix
V=1:length(P);
[~,inds] = histc(R,Pcum); 
T = V(inds);

%shape into output matrix
T=reshape(T,N,M);

%if desired, output plot
if nargin==4
    if strcmp(varargin{1},'plot')
        Pfreq=N*M*P/sum(P);
        figure;
        hold on
        hist(T(T>0),1:length(P))
        plot(Pfreq,'r-o')
        ylabel('Frequency')
        xlabel('P-vector Index')
        axis tight
        box on
    end
end

end
